/*
 * main.c
 *
 *  Created on: 16.02.2017
 *      Author: Micha3
 */
#include <avr/io.h>
#include <avr/interrupt.h>

#define temperature 0
#define pressure 1

volatile int32_t pw,tw;

volatile uint8_t j=0;
volatile uint16_t i=0;

void wait(void)
{
	volatile uint8_t x,y;

	for(x=0;x<5;x++)
	{
		for(y=0;y<20;y++)
		{

		}
	}
}

void send_start_cond(void)
{
	TWCR = (1<<TWSTA)| (1<<TWINT)|(1<<TWEN); // start transmisji
	while(!(TWCR & (1<<TWINT)));
	while((TWSR & 0xF8)!=0x08);
}

void send_restart_cond(void)
{
	TWCR = (1<<TWSTA)| (1<<TWINT)|(1<<TWEN); // restart transmisji
	while(!(TWCR & (1<<TWINT)));
	while((TWSR & 0xF8)!=0x10);
}



void send_data(uint8_t data)
{
	TWDR = data;
	TWCR = (1<<TWINT) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT)));
	while((TWSR & 0xF8)!=0x28);

}

void send_adress_w(uint8_t addr)
{
	TWDR = addr;
	TWCR = (1<<TWINT) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT)));
	while((TWSR & 0xF8)!=0x18);

}


void send_adress_r(uint8_t addr)
{
	TWDR = addr;
	TWCR = (1<<TWINT) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT)));
	while((TWSR & 0xF8)!=0x40);

}


void start_measure(uint8_t type)
{
	send_start_cond();

	send_adress_w((uint8_t)0xEE);

	send_data((uint8_t)0xF4); //BMP180 control register

	if(type == temperature)
		send_data((uint8_t)0x2E); // temp
	if(type == pressure)
		send_data((uint8_t)0x34); //cinienie - 0oss - ultra low mode

	//koniec transmiji
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
	wait();
}

uint16_t get_data(uint8_t reg_addr)// MSB
{
	uint16_t value=0;


	send_start_cond();

	send_adress_w((uint8_t)(0xEE));
	send_data((uint8_t)(reg_addr));


	send_restart_cond();
	send_adress_r((uint8_t)(0xEF));

	TWCR = (1<<TWINT) | (1<<TWEN) |(1<<TWEA);
	while (!(TWCR & (1<<TWINT)));
	while((TWSR & 0xF8)!=0x50);


	value = (uint16_t)(TWDR<<8);


	TWCR = (1<<TWINT) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT)));
	while((TWSR & 0xF8)!=0x58);

	value |= TWDR;

	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO); //stop
	wait();

	return value;
}



int32_t get_temperature(void)
{
	int32_t nctemperature=0;
	start_measure(temperature);
	nctemperature = (int32_t)(get_data(0xF6));
	return nctemperature ;

}

int32_t get_pressure(void)
{
	int32_t ncpressure=0;
	start_measure(pressure);
	ncpressure = (int32_t)(get_data(0xF6));
	return ncpressure ;

}
ISR(TIMER2_COMP_vect)
{
	i++;
	if(i==pw)
		PORTB &= ~(1<<PB0);
	if(i==160)		// wersja 300 -1100(rozdzielczosc 5hPa)
	//if(i==100) 	// wersja 950- 1050(rozdzielczosc 1hPa)
	{
		i=0;
		PORTB |= (1<<PB0);
	}
}

int main(void)
{

 int32_t t=0,p=0;
 int16_t AC1 = 0,AC2=0,AC3=0,B1=0,B2=0,MB=0,MC=0,MD=0;
 uint16_t AC4=0,AC5=0,AC6=0;
 int32_t x1=0,x2=0,x3=0,b3=0,b5=0,b6=0,nc_temp=0,nc_press=0;
 uint32_t b4=0,b7=0;

DDRD = 0xff;
DDRB |= (1<<PB0);
DDRC |=0b00001111;

TIMSK |= (1<<OCIE2);
TCCR2 |= (1<<WGM21)|(1<<CS20);


// sck = 10kHz
TWBR = 11;
TWSR |= ((1<<TWPS0) |(1<<TWPS1));

//read  coefficients
AC1 = (int16_t)(get_data(0xAA));
AC2 = (int16_t)(get_data(0xAC));
AC3 = (int16_t)(get_data(0xAE));
AC4 = (uint16_t)(get_data(0xB0));
AC5 = (uint16_t)(get_data(0xB2));
AC6 = (uint16_t)(get_data(0xB4));
B1 = (int16_t)(get_data(0xB6));
B2 = (int16_t)(get_data(0xB8));
MB = (int16_t)(get_data(0xBA));
MC = (int16_t)(get_data(0xBC));
MD = (int16_t)(get_data(0xBE));

nc_temp = get_temperature();
nc_press = get_pressure();

// obliczenia temperatury - mona pomin
x1 = (int32_t)((((int32_t)(nc_temp) - (uint32_t)(AC6)) * (uint32_t)(AC5)>>15));
x2 = (int32_t)(((int32_t)(MC)<<11)/(int32_t)(x1+(int32_t)(MD)));
b5 = x1+x2;
t= (b5+8)>>4;

// obliczenia cinienia - mona optymalizowac i zamiast dzielenia i mnozenia realizowac przesuniecia bitowe jw
b6 = b5 - 4000;
x1 = ((int32_t)(B2)*((b6*b6)/4096))/2048;
x2 = ((int32_t)(AC2)*b6)/2048;
x3 = x1+x2;
b3 = (((int32_t)(AC1))*4 + x3 + 2)/4;
x1 = ((int32_t)(AC3) * b6)/8192;
x2 = ((int32_t)(B1)*((b6*b6)/4096))/65536;
x3 = (x1+x2+2)/4;
b4 = (uint32_t)(((uint32_t)(AC4) * (uint32_t)(x3+32768))/32768);
b7 = (uint32_t)(((uint32_t)(nc_press)-b3)*50000);
if(b7<0x80000000)
	p=(b7*2)/b4;
else
	p=(b7/b4)*2;
x1 = (p/256)*(p/256);
x1 = (x1*3038)/32768;
x2 = (-7357*p)/32768;
p=p+(x1+x2+3791)/16;


sei(); // wczenie przerwa - dopiero, gdy gotowe s pierwsze wyniki


pw=((p/500)-60); 	// wersja 300 -1100(rozdzielczosc 5hPa)
//pw=((p/100)-950); // wersja 950- 1050(rozdzielczosc 1hPa)


	while(1)
	{
	//read  coefficients

	nc_temp = get_temperature();
	nc_press = get_pressure();

	x1 = (int32_t)((((int32_t)(nc_temp) - (uint32_t)(AC6)) * (uint32_t)(AC5)>>15));
	x2 = (int32_t)(((int32_t)(MC)<<11)/(int32_t)(x1+(int32_t)(MD)));
	b5 = x1+x2;
	t= (b5+8)>>4;

	b6 = b5 - 4000;
	x1 = ((int32_t)(B2)*((b6*b6)/4096))/2048;
	x2 = ((int32_t)(AC2)*b6)/2048;
	x3 = x1+x2;
	b3 = (((int32_t)(AC1))*4 + x3 + 2)/4;
	x1 = ((int32_t)(AC3) * b6)/8192;
	x2 = ((int32_t)(B1)*((b6*b6)/4096))/65536;
	x3 = (x1+x2+2)/4;
	b4 = (uint32_t)(((uint32_t)(AC4) * (uint32_t)(x3+32768))/32768);
	b7 = (uint32_t)(((uint32_t)(nc_press)-b3)*50000);
	if(b7<0x80000000)
		p=(b7*2)/b4;
	else
		p=(b7/b4)*2;
	x1 = (p/256)*(p/256);
	x1 = (x1*3038)/32768;
	x2 = (-7357*p)/32768;
	p=p+(x1+x2+3791)/16;

	pw=((p/500)-60);	// wersja 300 -1100(rozdzielczosc 5hPa)
//	pw=((p/100)-950); 	// wersja 950- 1050(rozdzielczosc 1hPa)



	}

}

